#include <stdio.h>
#include <qimage.h>

#include "constants.h"
#include "pmiroir.h"
#include "pmimage.h"

const QString PMImage::ol_fn_tail = "_pmiro.jpg";

PMImage::PMImage(PMiroir *p, const QString& d) : QObject((QObject*)p) {
  setupDir(d);
}

PMImage::~PMImage(void) {}

void PMImage::setupDir(const QString &d) {
  QRegExp isol(ol_fn_tail + "$");

  _dir = d;
  QDir dir(_dir);
  dir.setSorting(QDir::Name | QDir::IgnoreCase);
  dir.setFilter(QDir::Files);
  dir.setNameFilter("*.jpg *.png *.bmp");
  QStringList files = dir.entryList();
  for (QStringList::Iterator it = files.begin();
       it != files.end(); ++ it) {
    if ((*it).find(isol) == -1)
      _files += (*it);
  }

  _index = -1;
}

bool PMImage::loadImage(unsigned short int *buf) {
  return loadImage(buf, _index);
}

bool PMImage::loadNextImage(unsigned short int *buf) {
  return loadImage(buf, _index + 1);
}

bool PMImage::loadPrevImage(unsigned short int *buf) {
  return loadImage(buf, _index - 1);
}

bool PMImage::loadImage(unsigned short int *buf, int index) {
  if (index < 0) index = 0;
  if ((unsigned int)index >= _files.count()) 
    index = _files.count() - 1;

  PMiroir* p = (PMiroir*)parent();

  if (p->imageDir().isNull())
    return false;
  if (_dir != p->imageDir())
    return false;

  if (_index >= 0 && _index < (signed int)_files.count())
    emit fileClosed(_dir + "/" + _files[_index]);

  _index = index;
  _buf = buf;

  QString fn = _dir + "/" + _files[index];
  QImage img;
  if (!img.load(fn)) return false;
  
  if (img.width() == Height && img.height() == Width) {
    img = img.convertDepth(16);
    if (img.isNull()) return false;

    unsigned short int *ibuf = (unsigned short int*)img.bits();
    int x, y, fx, fy;

    for (y = 0; y < Height; y ++)
      for (x = 0; x < Width; x ++) {
        fx = Height - 1 - y;
        fy = x;
        buf[y * Width + x] = ibuf[fy * Height + fx];
      }
  } else {
    if (img.width() != Width || img.height() != Height) 
      img = img.smoothScale(Width, Height);

    img = img.convertDepth(16);
    if (img.isNull()) return false;

    unsigned long int *ibuf = (unsigned long int*)img.bits();
    unsigned long int *buf32 = (unsigned long int*)buf;

    for (int i = 0; i < Width * Height / 2; i ++)
      buf32[i] = ibuf[i];
  }

  emit fileLoaded(_dir + "/" + _files[index]);

  return true;
}

unsigned short int PMImage::pack_color(const QColor &c) {
  uint r, g, b;

  r = (c.red() >> 3);
  g = (c.green() >> 2);
  b = (c.blue() >> 3);

  return r << 11 | g << 5 | b;
}

